function [ws1,ws2,wu1,wu2] = fhn2_smflds(x,p,vinc)
%Stable and unstable manifolds of a saddle point of FitxHugh-Nagumo
%   Input: x coordinates of saddle point (from fhn_eq)
%          p parameters
%          vinc distance of initial conditions from x
%   Output: Manifold trajectories
jac = fhn2_jac(x,p);
[V,D] = eig(jac)
if (isreal(D(1,1)) && (D(1,1) < 0))
    sind = 1;
else
    sind = 2;
end
uind = 3-sind;
xs1 = x+vinc*[V(1,sind),V(2,sind)];
xs2 = x-vinc*[V(1,sind),V(2,sind)]; 
xu1 = x+vinc*[V(1,uind),V(2,uind)];
xu2 = x-vinc*[V(1,uind),V(2,uind)]; 
    
options = dopset('AbsTol',1e-14,'RelTol',1e-12,'EventTol',1e-14,'Events',1);
[ts1,ws1,te,ye,ie,stats] = dop853('fhn2d',[0 -300],xs1,options,p);
[ts2,ws2,te,ye,ie,stats] = dop853('fhn2d',[0 -300],xs2,options,p);
[tu1,wu1,te,ye,ie,stats] = dop853('fhn2d',[0 70],xu1,options,p);
[tu2,wu2,te,ye,ie,stats] = dop853('fhn2d',[0 70],xu2,options,p);

%figure(1)
%hold on
%plot(x(1),x(2),'o')
plot(ws1(:,1),ws1(:,2),'b',ws2(:,1),ws2(:,2),'b','LineWidth',1)
plot(wu1(:,1),wu1(:,2),'r',wu2(:,1),wu2(:,2),'r','LineWidth',1)

end

