# include "vfieldfunc.h"
# include <math.h>

/* 2D Fitzhugh-Nagumo equation
 Input:  (t, Y, p)
    t -- scalar (ignored)
    y -- phase-space vector [x; y]
    p -- parameter vector [a;b;c;e;equilibria;radius] 

 Output: 2-vector f 

   x_dot = (y+x-x^3/3)
   y_dot = -e (ax+b+cy)

 Three event functions

   On straight nullcline
   Near equilibrium 1
   Near equilibrium 3

*/

void vfieldfunc(unsigned n,  double t, double* y, double* p, double* f)
{

  f[0] = y[1] + y[0] -y[0]*y[0]*y[0]/3;
  f[1] = -p[3]*(p[0]*y[0]+p[1]+p[2]*y[1]);
} 

void eventfunc (unsigned ndims, double t, double *y, double *p, 
	int ievent, double &value, int &isterminal, int &direction)
{
	switch (ievent)
	{
	case 0:
		value = p[0]*y[0]+p[1]+p[2]*y[1];
		isterminal = 1;
		direction = 1;
		break;
/*	case 1:
		value = (y[0]-p[4])*(y[0]-p[4])+(y[1]-p[5])*(y[1]-p[5])-p[10];
		isterminal = 1;
		direction = -1;
		break;
	case 2:
		value = (y[0]-p[8])*(y[0]-p[8])+(y[1]-p[9])*(y[1]-p[9])-p[10];
		isterminal = 1;
		direction = -1;
		break;
   case 3:
 		value = y[0]*y[0] + y[1]*y[1] -10;
		isterminal = 1;
		direction = 0;
		break; */
}
} 
