# include "vfieldfunc.h"
# include <math.h>

/* 2D Fitzhugh-Nagumo equation
 Input:  (t, Y, p)
    t -- scalar (ignored)
    y -- phase-space vector [x; y]
    p -- parameter vector [a;b;c;e;equilibria;radius] 

 Output: 2-vector f 

   x_dot = (y+x-x^3/3)
   y_dot = -e (ax+b+cy)

 Three event functions

   On straight nullcline
   Near equilibrium 1
   Near equilibrium 3

*/

void vfieldfunc(unsigned n,  double t, double* y, double* p, double* f)
{

  f[0] = y[1] + y[0] -y[0]*y[0]*y[0]/3;
  f[1] = -p[3]*(p[0]*y[0]+p[1]+p[2]*y[1]);
} 

void eventfunc (unsigned ndims, double t, double *y, double *p, 
	int ievent, double &value, int &isterminal, int &direction)
{
	switch (ievent)
	{
	case 0:
		value = p[0]*y[0]+p[1]+p[2]*y[1];
		isterminal = p[4];
		direction = p[5];
		break;
}
} 
