%Saddle -node of periodic orbits in the Fitzhugh-Nagumo equation
%   f[0] = y[1] + y[0] -y[0]*y[0]*y[0]/3;
%   f[1] = -p[3]*(p[0]*y[0]+p[1]+p[2]*y[1]);
%
format long
%The parameters
p = [1, 0.05, 1.2, 0.3921]
%For plotting
figure(1)
clf
hold on
axis([-2,2,-1,1])
%Equilibrium points and nullclines
eqm = fhn_eq(p)
fhn_nullcline(p);
%Stable and unstable manifolds
[ws1,ws2,wu1,wu2] = fhn2_smflds(eqm(3,:),p,1e-5);

%Plot 100 trajectories to first return with increasing coordinate for
%cross-section that determines linear nullcline
xn = 100;
%x coordinate increment between intial points
xinc = 1e-5;
%matrix of initial points 
xin = -1.042 + xinc*[1:100];
yin = (-p(1)*xin-p(2))./p(3);
pin = [xin',yin'];
%For output - checking that return is to correct region
pin2= [];
pout = [];
options = dopset('AbsTol',1e-14,'RelTol',1e-12,'MaxStepSize',0.01,'MaxSteps',1e7,'EventTol',1e-14,'Events',1);
%Compute the trajectories and store initial and final points
for j=1:xn
  [ts,pts,te,ye,ie,stats] = dop853('fhn_el2',[0 2000],pin(j,:),options,p);
  plot(pts(:,1),pts(:,2),'g')
  if (pts(end,1) <-0.75)
      pin2 = [pin2;pin(j,:)];
  pout = [pout;pts(end,:)];    
  end
end
%Compute derivatives for return
%difference of x-coordinate endpoints
xd = pin2(:,1) -pout(:,1);
%Plot x-coordinate differences vs. initial values of x
figure(2)
clf 
hold on
plot(pin2(:,1),xd,'x',[xin(1),xin(end)],[0,0]);
%
% plot(pin2(:,1),xd,'x')
% plot([pin2(1,1),pin2(end,1)],[0,0],'k')
% %Closest return to a fixed point
% [resid,ind] = min(abs(pin2-pout))
% ind = ind(1);
% %Compute first and second derivatives of x-coordinate endpoint differences
% x0 = pin2(ind,1);
% y0 = xd(ind,1)
% dx0 =  (xd(ind+1)-xd(ind-1))/(2*xinc)
% dxx0 = (xd(ind+1)-2*xd(ind)+xd(ind-1))/(xinc^2)
% %Quadratic fit to return
% qfit = y0 + dx0*(pin2(:,1)-x0) + dxx0*(pin2(:,1)-x0).^2/2;
% %Plot quadratic green
% plot(pin2(:,1),qfit,'go')
% %Plot residual red
% plot(pin2(:,1),xd-qfit,'ro')
% % Increment parameter p(4)
% poutp = [];
% pinp = [];
% pinc = 1e-7;
% pdp = p+pinc*[0;0;0;1];
% %Compute trajectories for perturbed p(4)
% for j=1:3
% [ts,pts,te,ye,ie,stats] = dop853('fhn_el2',[0 2000],pin(j+ind-2,:),options,pdp);
% pinp = [pinp;pin(j+ind-2,:)];
% poutp = [poutp;pts(end,:)]; 
% end;
% %Compute differences and derivatives for perturbed parameters
% xdp = poutp(:,1) - pinp(:,1);
% dp0 = (poutp(2,1)-pout(2,1))/pinc;
% dxp0 =  (xdp(3) - xdp(1))/(pinp(3,1)-pinp(1,1));
% dxp0p = (dxp0-dx0)/pinc;
% %Jacobian for defining function (\sigma - id,\sigma' - 1)
% jac = [[dx0,dp0];[dxx0,dxp0]]
% %
% %One Newton step
% xpnew = [x0;p(4)] - jac\[xd(ind);dx0];
% pnew = [p(1:3);xpnew(2)];
% xynew = [xpnew(1),(-p(1)*xpnew(1)-p(2))/p(3)];
% %Trajectory at new initial points and parameter - compute return and
% %derivative
% [ts,pts,te,ye,ie,stats] = dop853('fhn_el2',[0 2000],xynew,options,pnew);
% xl = xpnew(1)-xinc;
% yl = (-p(1)*xl-p(2))/p(3);
% [tsl,ptsl,te,ye,ie,stats] = dop853('fhn_el2',[0 2000],[xl,yl],options,pnew);
% xr = xpnew(1)+xinc;
% yr = (-p(1)*xr-p(2))/p(3);
% [tsr,ptsr,te,ye,ie,stats] = dop853('fhn_el2',[0 2000],[xr,yr],options,pnew);
% dxpert0 = (ptsr(end,1)-ptsl(end,1))/(2*xinc) - 1
% %Compare values before and after Newton step
% be0 = [pin2(ind,:);pout(ind,:)]
% benew = [pts(1,:);pts(end,:)]
% %Note that sigma' - 1 has decreased by a factor of approximately 100 after
% %the Newton step (from dx0 to dxpert0)
% %The increment of the newton step was 
% newt_delta = jac\[xd(ind);dx0]
