#include <vector>
#include "mex.h"
#include "globals.h"
#include "integrator.h"
#include "typedefs.h"
#include "utilfuncs.h"
#include "EventObj.h"
#include "memory.h"

int checkinput(int nrhs, const mxArray *prhs[])
{
	int result = 0;

	switch (nrhs)
	{
	case 2:
		if (mxIsDouble(prhs[0])
			&& mxIsDouble(prhs[1]))
			result = 1;
		break;

	case 3:
		if (mxIsDouble(prhs[0])
			&& mxIsDouble(prhs[1])
			&& (mxIsDouble(prhs[2]) || mxIsStruct(prhs[2])))
			result = 1;
		break;

	case 4:
		if (mxIsDouble(prhs[0])
			&& mxIsDouble(prhs[1])
			&& mxIsStruct(prhs[2])
			&& mxIsDouble(prhs[3]))
			result = 1;
		break;

	default:
		result = 0;
		break;
	}
	
	return (result);
}

void outputfun(double t, double* y, int n) 
{
	vector<double> vy(y,y+n);

	// do a push_back if t is within machine precision of wanted time-step. It not use contd8
	points.push_back(vy);
	timev.push_back(t);

	return;
}

// Refinment function, called from solout()
void refine(int n, double xold, double x) 
{ 
	int i, j;
	int r=options.Refine;

	double dx = (x - xold) / (r+1);
	double t = xold;

	if (yout.empty()) {    // init yout;
		yout.assign(n,0.0);
		//for (i=0; i<n; i++)
		//	yout.push_back(0.0);
	}

	for (j=0; j<r ; j++) {
		t = t+dx;
		for (i=0; i<n; i++)
			yout[i] = contd8(i, t);

		outputfun( t, &(yout[0]), n);
	}
}

// Branch-out function, called after each accepted point
void dopri_solout(long nr, double xold, double x, double* y, unsigned n, long nmax, int* irtrn)
{
	/* Note on implementing outputs at fixed time-stamps:
			This is the place to do it. If xold <= xdesired <= x
			then use the contd8 to find y[] at xdesired and pass that
			to the outputfun. */

	*irtrn = 0;  // stop-flag unset

	// Just output the initial point
	if (timev.empty()) {
		outputfun(x, y, n);
		return;
	}

	// Check for events 
	if ( options.Events > 0 )
	{
		EventFlag eventoutcome = events.testforevents(x, y);
		*irtrn = (eventoutcome==TerminalEvent)?-1:0;	// If there was any terminal event, stop integration
	}

	// Do refinement
	if (options.Refine > 0)
		refine(n, xold, x);

	// Output this point
	if ( !(*irtrn < 0) )	  //JMG 5/27/2011: Replace final point by terminal event
		{ outputfun(x, y, n); }
		else
		{
			points.push_back(ye.back());
		 	timev.push_back(te.back());
		}

	// Check limit on number of points
	if (timev.size() >= nmax)
		*irtrn = -1;             // set stop-flag
}

